Module: Ronin

Defined in:
lib/ronin/asm/version.rb,
lib/ronin/asm/asm.rb,
lib/ronin/asm/os/os.rb,
lib/ronin/asm/config.rb,
lib/ronin/asm/program.rb,
lib/ronin/asm/os/linux.rb,
lib/ronin/asm/register.rb,
lib/ronin/asm/archs/x86.rb,
lib/ronin/asm/shellcode.rb,
lib/ronin/asm/os/freebsd.rb,
lib/ronin/asm/syntax/att.rb,
lib/ronin/asm/archs/amd64.rb,
lib/ronin/asm/instruction.rb,
lib/ronin/asm/syntax/intel.rb,
lib/ronin/asm/syntax/common.rb,
lib/ronin/asm/memory_operand.rb,
lib/ronin/asm/immediate_operand.rb

Overview

Ronin ASM - A Ruby DSL for crafting Assembly programs and Shellcode.

Copyright (c) 2007-2013 Hal Brodigan (postmodern.mod3 at gmail.com)

This file is part of Ronin ASM.

Ronin is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

Ronin is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with Ronin. If not, see http://www.gnu.org/licenses/

Defined Under Namespace

Modules: ASM